const { app, Menu, BrowserWindow, webContents, ipcMain } = require('electron');
const path = require('path');
const url = require('url');
const Config = require('electron-config');
const config = new Config();
const menuConfigFn = require('./menu.config');

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;

function createWindow() {
  // Create the browser window.
  let opts = { show: false, frame: false};
  Object.assign(opts, config.get('winBounds'));
  mainWindow = new BrowserWindow(opts);

  // set menu
  const menuConfig = menuConfigFn();
  const menu = Menu.buildFromTemplate(menuConfig);
  Menu.setApplicationMenu(menu);
  // and load the index.html of the app.
  mainWindow.loadURL(url.format({
    pathname: path.join(__dirname, 'index.html'),
    protocol: 'file:',
    slashes: true
  }));

  mainWindow.once('ready-to-show', mainWindow.show);

  // Open the DevTools.
  // mainWindow.webContents.openDevTools()

  mainWindow.on('close', () => {
    config.set('winBounds', mainWindow.getBounds());
  });

  mainWindow.on('closed', function () {
    mainWindow = null;
  });

  mainWindow.webContents.on('new-window', function (e, url, frameName, disposition, options, additionalFeatures) {
    const allWins = webContents.getAllWebContents();
    const opt = mainWindow.getBounds();
    options.width = opt.width;
    options.height = opt.height;
    options.x = opt.x;
    options.y = opt.y;
    e.preventDefault();

    const existWins = allWins.filter(w => w.getURL() === url);

    if (existWins.length > 0) {
      existWins[0].focus();
    } else {
      let childWindow = new BrowserWindow({frame: true, show: false});
      childWindow.once('ready-to-show', childWindow.show);
      childWindow.on('closed', function () {
        childWindow = null;
      });
      childWindow.loadURL(url);
    }
  });
}

ipcMain.on('min', e => mainWindow.minimize());
ipcMain.on('max', e => {
  if (mainWindow.isFullScreen()) {
    mainWindow.setFullScreen(false);
  } else {
    mainWindow.setFullScreen(true);
  }
});
ipcMain.on('close', e => {
  mainWindow.close()
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', function () {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit();
  }
})

app.on('activate', function () {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) {
    createWindow();
  }
});

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.
